@@ -54,9 +54,8 @@
 #'
 #'   }
 #' }
-#' @return if \code{plot = FALSE} a list is returned containing a data.frame
-#' with the new sample coordinates and a vector of length x that corresponds to
-#' each sample's group.
+#' @return if \code{plot = FALSE} a data.frame is returned containing the new
+#' sample coordinates and the group they are assigned to.
 #'
 #' @examples
 #'
@@ -127,18 +126,21 @@ sinaplot <- function(x,
     x <- .getXcoord(x, groups, yBins, xSpread, groupwiseScale, neighbLimit,
                     adjust, method)
 
+    if (is.null(labels))
+        labels <- levels(groups)
+
     newGroups <- factor(rep(levels(groups), unlist(lapply(x, nrow))))
 
-    if (!is.null(labels))
-        labs <- labels
-    else
-        labs <- levels(groups)
-
+    #unlist
     x <- do.call(rbind, x)
 
-    if (plot == TRUE){
+    x$groups <- newGroups
 
-        x$groups <- newGroups
+    #order the data frame based on the input order
+    x <- x[order(x$idx), ]
+    x <- x[, c("x", "y", "groups")]
+
+    if (plot){
 
         p <- ggplot2::ggplot(ggplot2::aes_string(x = 'x', y = 'y',
                                                  color = 'groups'), data = x) +
@@ -147,7 +149,9 @@ sinaplot <- function(x,
         if (bw)
             p <- p + ggplot2::theme_bw()
 
-        p <- p + ggplot2::scale_x_discrete(limits = labs) +
+        p <- p + ggplot2::scale_x_continuous(breaks = 1:length(labels),
+                                             labels = labels,
+                                             minor_breaks = NULL) +
             ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45,
                                                                hjust = 1))
         p <- p + ggplot2::xlab("") + ggplot2::ylab(ylab) +
@@ -158,7 +162,7 @@ sinaplot <- function(x,
         p
 
     } else
-        list(x, newGroups)
+        x
 }
 
 .binY <- function(data, yFraction) {
@@ -197,12 +201,16 @@ sinaplot <- function(x,
         densities <- c()
     }
 
+    #keep an index of the original data order
+    idx <- 1:length(data)
+
     for (j in 1:ngroups){
 
         #extract samples per group and store them in a data.frame
-        cur_xyArray <- as.data.frame(cbind(rep(j, sum(groups == levels(groups)[j])),
-                                           as.numeric(data[groups == levels(groups)[j]])))
-        colnames(cur_xyArray) <- c("x", "y")
+        keep <- groups == levels(groups)[j]
+        cur_xyArray <- as.data.frame(cbind(rep(j, sum(keep)),
+                                           as.numeric(data[keep]), idx[keep]))
+        colnames(cur_xyArray) <- c("x", "y", "idx")
 
         #find the densiest neighbourhood in the current group and compare it
         #with the global max
